#include <assert.h>
#include <sys/mman.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sched.h>
#include <stdlib.h>
#include <limits.h>
#include "baudtab.h"
#include "canctl.h"
#include "peekpoke.h"

void sbuslock(void);
void sbusunlock(void);
void sbuspreempt(void);
unsigned int ts4800_can_peek8(int adr);
void ts4800_can_poke8(int adr, unsigned int dat);
void ts4800_can_peekstream(int adr, unsigned char *buf, int n);
void ts4800_can_pokestream(int adr, unsigned char *buf, int n);

extern int gotHUP;
static volatile unsigned int *cvspiregs, *cvgpioregs;

void _init_ts4800() {
}

static int devmem = -1;
static volatile unsigned char *Base;
static const int CANAddr = 0;
static const int CANData = 2;

void init_ts4800(int port) {
  volatile unsigned short *en;

  en = map_phys(0xB0010010,&devmem);
  assert(en);
  if (port == 0) {
    en[0] |= 1;
    Base = map_phys(0xB00A0000,&devmem);
  } else { // port == 1
    en[0] |= 2;
    Base = map_phys(0xB00B0000,&devmem);
  }
  assert(Base);

  buslock = sbuslock;
  busunlock = sbusunlock;
  buspreempt = sbuspreempt;
  can_peek8 = ts4800_can_peek8;
  can_poke8 = ts4800_can_poke8;
  can_peekstream = ts4800_can_peekstream;
  can_pokestream = ts4800_can_pokestream;

  irq_num = 30; // ???
  baudparms = baudparms25;
}

void sbuslock(void) {
  if (gotHUP) {
    _init_ts4800();
    gotHUP = 0;
  }
}

void sbusunlock(void) {
}


void sbuspreempt(void) {
}

unsigned int ts4800_can_peek8(int adr) {
  Base[CANAddr] = adr;
  return Base[CANData];
}


void ts4800_can_poke8(int adr, unsigned int dat) {
  Base[CANAddr] = adr;
  Base[CANData] = dat;
}


void ts4800_can_peekstream(int adr, unsigned char *buf, int n) {
  Base[CANAddr] = adr;
  while (n-- > 0) {
    *buf++ = Base[CANData];
  }
}


void ts4800_can_pokestream(int adr, unsigned char *buf, int n) {
  Base[CANAddr] = adr;
  while (n-- > 0) {
    Base[CANData] = *buf++;
  }
}

